 /*
 * The input routines.  This program runs as a child process to the
 * Pcomm program.
 */

#define CLIST 1024

#include <curses.h>
#include <stdio.h>
#include <signal.h>
#include <setjmp.h>
#include <fcntl.h>

#include "config.h"
#ifdef SHAREDMEM
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#endif /* SHAREDMEM */
#define MAIN
#include "misc.h"
#include "status.h"
#include "param.h"

#ifdef	M_TERMINFO
#include	<term.h>
#endif

#include "vcs.h"
#include "btx.h"
#include "btx_table.h"

jmp_buf i_jmp;
int vcs_param[NUM_VCS][5];		/* positional parameters */
int vcs_opt[NUM_VCS][10];		/* options unique to each vcs */
int vcs_codes[NUM_VCS][VCS_SIZE];	/* the vcs codes */
int vcs_leadin[NUM_VCS];		/* unique list of lead-in characters */
int num_leadin;				/* length of lead-in list */
int hold, max_row, max_col, skip_row;

FILE *lprfp;

struct STATUS *status;

#ifdef SHAREDMEM
#define VROW	status->row
#define VCOL	status->col
#define LVS	status->vs
#else /* SHAREDMEM */
int		VROW, VCOL;
char LVS[MAX_ROW][MAX_COL+2];
struct STATUS s;
#endif /* SHAREDMEM */

/*
 * Read the serial port and write the characters to the screen.  Watch
 * for signals from the parent process to toggle the fancy options.
 * Writes the characters received to a virtual screen buffer.
 */
#ifdef	M_TERMCAP
#define	cbreak	crmode
#endif

main(argc, argv)
int argc;
char *argv[];
{
	
	FILE *popen();
	unsigned int c1, c2;
	int	cb, rc, rc1;
	char	str[2];
	int got_sig(), shut_input();
	unsigned char c;
	char *strcpy();
	FILE *fopen();

	char	*term, *getenv(), *tgetstr();
	int	s1,s2,s3,s4;
	int	zz;
	int	testbyte=0;
	int 	send_str();
	int  btx_conv();
	unsigned short 	btx_locate();
	void btx_cls();
	void btx_log_cls();
	void btx_log();

	void _exit(), exit(), vcs_table();
	int  i_exit(), btx_prot();


#ifdef SHAREDMEM
	int shm_id;
	char *shmat();
	void perror();
#endif /* SHAREDMEM */

	char	*i;
	char	tcbuf[1024], tb[1024], *t;

	term 		= (char *)NULL;
	term 		= getenv("TERM");
	
	if(term == (char *)NULL){
		rc=putenv("TERM=ansi");
		term = getenv("TERM");
	}
#ifdef	M_TERMCAP
	tgetent( tb, term);		/* get terminal			*/
	t = tcbuf;

	btx_CH = tgetstr("ho", &t);	/* read seq. cursor_home	*/
	btx_CL = tgetstr("cl", &t);	/* read seq. clear screen	*/
	btx_CE = tgetstr("ce", &t);	/* read seq. delete end of line */
	btx_SO = tgetstr("so", &t);	/* read seq. invers on		*/
	btx_SE = tgetstr("se", &t);	/* read seq. invers off		*/
	btx_US = tgetstr("us", &t);	/* read seq. underline on	*/
	btx_UE = tgetstr("ue", &t);	/* read seq. underline off	*/
#else
/*
	setupterm(getenv("TERM"), 1, &i);
*/

	btx_SE = exit_attribute_mode;
	btx_US = enter_underline_mode;
	btx_UE = exit_underline_mode;
#endif

	ACKx = ACK1;			/* set default start ACKx	*/
	
	DEBUG=ON;			/* set debug log on/off		*/
	PROT=ON;			/* set protokoll log on/off	*/

	btx_XS = 0;			/* set row	start log	*/
	btx_XE = 25;			/* set row	end   log	*/
	btx_YS = 0;			/* set colum	start log	*/
	btx_YE = 80;			/* set colum 	end   log	*/
	
	initscr();			/* init curses pagage		*/
#ifdef	COLOR
	start_color();
#endif
	cbreak();
	noecho();
	nonl();
					/* set the trap for the signals */
	signal(SIGALRM, SIG_IGN);
	signal(SIGHUP,  shut_input);
	signal(SIGQUIT, SIG_IGN);
	signal(SIGUSR1, got_sig);
	signal(SIGUSR2, got_sig);
	signal(SIGINT,  got_sig);

	signal(SIGKILL, shut_input);	
	signal(SIGTERM, shut_input);
					/* unbuffered output */
	setbuf(stdout, (char *) NULL);

					/* for the curious... */
	
	if (argc == 1) {
		fprintf(stderr,"This is the input routine for Pcomm program\n");
		fprintf(stderr,"It is not designed to be run as a separate\n");
		i_exit(1);
	}
#ifdef SHAREDMEM
	shm_id = atoi(argv[1]);
	status = (struct STATUS *) shmat(shm_id, (char *) 0, 0);
	if ((int) status == -1) {
		perror("shmat");
		_exit(1);
	}
#else /* SHAREDMEM */
	status = &s;
#endif /* SHAREDMEM */
					/* load the VCS table */
	vcs_table();
	if (max_row > MAX_ROW)
		max_row = MAX_ROW;
	if (max_col > MAX_COL)
		max_col = MAX_COL;


					/* parse the command line */
#ifndef SHAREDMEM
	status->fd 	= atoi(argv[1]);	/* fd-number (line)	*/
	status->add_lf 	= atoi(argv[2]);	/* add-line feed on cr 	*/
	status->log 	= atoi(argv[3]);	/* logfile write on/off */
	status->print 	= atoi(argv[4]);	/* printer on/off	*/
	status->btx   	= atoi(argv[5]);	/* btx-mode on/off	*/
	strcpy(status->log_path, argv[6]);	/* logfile-name		*/
	strcpy(status->log_macro, argv[7]);	/* macro input-file	*/
	strcpy(status->vs_path, argv[8]);	/* tmp page directory	*/
#endif /* SHAREDMEM */

	if(status->btx == 2)	btx_CRON =ON;	/* set cron mode	    */
					/* der cron modus gibt keine        */
					/* ausgaben auf den bildschirm aus  */	
	
	if(btx_CRON) term_l_mode();  	/* set ioctl param for cron mode    */	
	
	strcpy(btx_MFILE, status->log_macro);	
					/* wird die input routine mit macro */
					/* gestartet, so soll das logfile   */
					/* nur durch das macro ON/OFF	    */
					/* gesteuert werden (default OFF)   */
	/*
	if( strncmp(btx_MFILE,"NONE",4) != 0)
		btx_L_MACRO=OFF;
	*/
	btx_TFI  = 0;
	btx_TRD  = 0;
	btx_TSW	 = 0;
	btx_TSW_dn=0;
	skip_row = 1;

#ifdef SHAREDMEM
	if (status->clr)
		skip_row = 1;
#else /* SHAREDMEM */
					/* read previous screen */
	if (!access(status->vs_path, 0))
		read_vs();
	else
		skip_row = 1;
#endif /* SHAREDMEM */

	hold = 0;
					/* start up file pointers */
	lprfp = (FILE *)NULL;
	logfp = (FILE *)NULL;


	switch (setjmp(i_jmp)) {
		case 0:			/* no signal */
			break;
		case 1:			/* toggle the data logging */
			status->log = status->log ? 0 : 1;
			break;
		case 2:			/* toggle the printer only no btx */
			if(!status->btx){
				status->print = status->print ? 0 : 1;
			}
			break;
		case 3:			/* suspend the input */
			hold = hold ? 0 : 1;
#ifndef SHAREDMEM
			if (hold)
				write_vs();
#endif /* SHAREDMEM */
			break;
		case 4:			/* clean up and go home */
			if (status->log)
				fclose(logfp);

			if (status->print) {
				putc('\f', lprfp);
				pclose(lprfp);
			}
#ifdef SHAREDMEM
					/* detach shared memory */
			shmdt((char *) status);
#endif /* SHAREDMEM */
			_exit(0);
			break;
	}
					/* any signal will awaken pause() */
	if (hold)
		pause();

	if((DEBUG) || (PROT)){
		if(( flink = fopen("link.log","wb"))== (FILE *)NULL){
			fprintf(stderr, "Can't open link.log\n");
			i_exit(-1);
		}
	}
					/* open or close the printer */
	if (status->print && lprfp == (FILE *)NULL)
		lprfp = popen(LPR, "w");


	if (!status->print && lprfp != (FILE *)NULL) {
		putc('\f', lprfp);
		pclose(lprfp);
		lprfp = (FILE *)NULL;
	}
					/* open or close the log file */
	if (status->log && logfp == (FILE *)NULL) {
		if (strcmp(status->log_path, "NONE")!=0) {
			if(!(logfp=fopen(status->log_path,"wb"))){
				status->log = 0;
				if(DEBUG)
				fprintf(stderr,"<OPEN-ERROR M-LOG>");
			}else{
				status->log = 1;
				if(DEBUG) fprintf(flink,"<LOG-PAGE-ON>");
			}
		}
		else
			status->log = 0;
	}
	if (!status->log && logfp != (FILE *)NULL) {
		fclose(logfp);
		logfp = (FILE *) NULL;
		if(DEBUG)	fprintf(flink,"<LOG-PAGE-OFF>");
	}

#ifdef SHAREDMEM
	if (status->clr) {
		status->clr = 0;
		vs_clear();
	}
#else /* SHAREDMEM */
					/* clear if vs_path doesn't exist */
	if (access(status->vs_path, 0)){
		vs_clear();
	}
#endif /* SHAREDMEM */
	/*
	 * The very first screen we see after dialing has the "Connected to..."
	 * message at row 0, therefore we start our virtual screen at row 1.
	 */
	if (skip_row) {
		skip_row = 0;
		VROW = 1;
	}
	if ( status->btx ){
		send_prot(DCT);		/* send him DCT	*/
		btx_log_cls();
	}


	while (1) {

		if( btx_MERKBYTE ){
			c2 = btx_MERKBYTE;
			btx_MERKBYTE = NUL;
		}else{

			/* read byte from tty port serial bufferd	*/

			if ( readbyte(&c1)  == 0)
				continue;
			
			c = (unsigned char) c1;

			/* Emulation wenn Schalter status->btx */

			if(status->btx){
				if(!(c2=btx_conv(c))){
					continue;
				}
			}
		}	

		if(status->btx) {
			if(btx_GSET==0)		c = g0tb[c2];
			if(btx_GSET==1)		c = g0tb[c2];
			if(btx_GSET==2)		c = g2tb[c2];
			if(btx_GSET==3)		c = g3tb[c2];
			if(btx_GSET==4)		c = g4tb[c2];
			if(btx_GSET==5){
					if(DEBUG) fprintf(flink,"(%x)", c2);
					c = (int)NUL;	/* DRCS	  	  */
					btx_lbyte=NUL;	/* No RPT on DRCS */
			}
			/* if == 2 then BTX-Terminal not use Blockgrafik */
			if((status->btx == 3) && (c < 23 | c > 128)) 	c='.';

			if( c ){		/* merker nur wenn char	  */
				btx_lbyte=c;	/* merker fuer RPT repeat */
			}
		}
					/* send to logfile */
		if ((status->log) && (!status->btx)) {
			if (c == '\r' && status->add_lf){
				if(logfp!=(FILE *)NULL)
					fprintf(logfp,"\n");
			}
					/* no carriage returns in logfile */
			if ((c != '\r')  && (logfp != (FILE *)NULL))
				fprintf(logfp, "%c", c);
		}
				/* send to printer too and only no btx */
		if ((status->print) && (!status->btx))
			putc(c, lprfp);

		cb= (int) c;
		if( cb > 0 ){
			if(status->btx){
				if(!btx_CRON){	/* batch cron mode */
					sprintf(str, "%c", (chtype)cb);
					addstr( str );
					rc1=refresh();
					vs_putchar ( cb );
				}
			}else{
				putchar( cb );
				vs_putchar( cb );
			}
			if(DEBUG)	fprintf(flink,"%c", cb);

			if(status->btx){
				btx_log( cb );
				btx_Y++;
				if(btx_Y >= 40){
					btx_Y=0; btx_X++;
					if((btx_DBH) && (SC==2)) btx_X++;
					btx_DBS = OFF;
					btx_DBH = OFF;
					btx_locate(btx_X, btx_Y);	
					if(DEBUG) fprintf(flink,"<LFT[%d][%d]>",								btx_X,btx_Y);	
				}
				if((btx_DBS) && (!btx_ATR)){
					if(!btx_CRON){
						sprintf(str,"%c",(chtype)SPACE);
						addstr( str );
						refresh();
						vs_putchar( (int)SPACE );
					}
					btx_log( (int)SPACE );
					if(DEBUG) fprintf(flink,"%c",SPACE);
					btx_Y++;
				}
			}
		}
					/* add LF to CR? */
		if ((c == '\r' && status->add_lf) && (!status->btx))
			putchar('\n');
	}
}

int
singleshift( byte, cmd )
char	byte;
int	cmd;
{
	switch( cmd )
	{
		case 1:
			return( g1tb[byte] );
			break;
		case 2:
			return( g2tb[byte] );
			break;
		case 3:
			return( g3tb[byte] );
			break;
		case 4:
			return( g4tb[byte] );
			break;
		default:
			return( byte );     
			break;
	}
return( byte );
}



term_l_mode()
{

	struct termio tbuf;

	ioctl( status->fd, TCGETA, &tbuf );
	tbuf.c_iflag	&=	~ICRNL;
	ioctl( status->fd, TCSETA, &tbuf );
	
	return;
}

/*
*	Sende ein Byte ueber die geoeffnete tty nn	
*/

int
send_prot( c )
char	*c;
{
int	rc=0;
	rc=write(status->fd , &c , 1);
	if((DEBUG) && (rc != 1))	fprintf(flink,"<WR[%d]>",rc);	
	return(rc);
}

int
send_string( str )
char	str[255];
{
int	rc=0;
int	z=0;
	z = strlen(str);

	rc=write(status->fd, str, z);
	if((DEBUG) && (rc != z))	fprintf(flink,"<WRS[%d][%d]>",z,rc);	
	return(rc);
}


/*
 * Figure out which signal we just received, and fix the return code of
 * the setjmp function above to the proper value.
 */

int
got_sig(sig)
int sig;
{
	void longjmp();
	switch (sig) {
		case SIGUSR1:
			signal(SIGUSR1, got_sig);
			longjmp(i_jmp, 1);
		case SIGUSR2:
			signal(SIGUSR2, got_sig);
			longjmp(i_jmp, 2);
		case SIGINT:
			signal(SIGINT, got_sig);
			longjmp(i_jmp, 3);
		case SIGTERM:
			signal(SIGTERM, got_sig);
			longjmp(i_jmp, 4);
	}
}

int
shut_input()
{
	if(btx_CRON){
		fprintf(stderr, "\nChild Process wurde abgebrochen (kill)\n");
	}else{
	}
	i_exit(-1);
}

void
vs_pos( xx, yy )
{
	VROW = xx;
	VCOL = yy;
	return;
}

/*
 * Put a character in the virtual screen.  This routine saves incoming
 * characters in a two dimensional buffer designed to mimic the real
 * screen.
 */

int
vs_putchar(c)
char c;
{
	register int j, i;
	int tab_stop;

	switch (vcs_filter(c)) {
		case MAYBE:		/* wait and see... */
			break;
		case 256+HOME:		/* home virtual screen "cursor" */
			VROW = 0;
			VCOL = 0;
			break;
		case 256+CLR_EOL:	/* clear to end of line */
			for (i=VCOL; i<max_col; i++)
				LVS[VROW][i] = ' ';
			break;
		case 256+CLR_EOS:	/* clear to end of screen */
			for (j=VCOL; j<max_col; j++)
				LVS[VROW][j] = ' ';
			for (i=VROW+1; i<max_row; i++) {
				for (j=0; j<max_col; j++)
					LVS[i][j] = ' ';
			}
			break;
		case 256+CLEAR:		/* clear all and home "cursor" */
			for (i=0; i<max_row; i++) {
				for (j=0; j<max_col; j++)
					LVS[i][j] = ' ';
			}
			VROW = 0;
			VCOL = 0;
			break;
		case 256+MV_UP:		/* move "cursor" up */
			VROW--;
			if (VROW < 0)
				VROW = 0;
			break;
		case 256+MV_DOWN:	/* move "cursor" down */
			VROW++;
			if (VROW >= max_row)
				VROW = max_row -1;
			break;
		case 256+MV_RIGHT:	/* move "cursor" right */
			VCOL++;
			if (VCOL >= max_col)
				VCOL = max_col -1;
			break;
		case 256+MV_LEFT:	/* move "cursor" left */
		case BS:		/* non destructive back space */
			VCOL--;
			if (VCOL < 0)
				VCOL = 0;
			break;
		case 256+MV_DIRECT:	/* direct cursor movement */
			VROW = vcs_param[MV_DIRECT][0];
			VCOL = vcs_param[MV_DIRECT][1];

					/* if "add one" and "decimal" */
			if (vcs_opt[MV_DIRECT][0] && vcs_opt[MV_DIRECT][1]) {
				VROW--;
				VCOL--;
			}
					/* if "character" */
			if (vcs_opt[MV_DIRECT][2]) {
					/* if "add offset" */
				if (vcs_opt[MV_DIRECT][3]) {
					VROW -= vcs_opt[MV_DIRECT][5];
					VCOL -= vcs_opt[MV_DIRECT][5];
				}
					/* if "subtract offset" */
				if (vcs_opt[MV_DIRECT][4]) {
					VROW += vcs_opt[MV_DIRECT][5];
					VCOL += vcs_opt[MV_DIRECT][5];
				}
				VROW--;
				VCOL--;
			}
			break;
		case 0:
		case 7:			/* skip NULL and "bell" character */
			break;
		case '\t':		/* tab character */
			tab_stop = VCOL + 8 - (VCOL % 8);
					/* if wrap around */
			if (tab_stop >= max_col) {
					/* spaces up to eol */
				for (; VCOL<max_col; VCOL++)
					LVS[VROW][VCOL] = ' ';
				VROW++;
				if (VROW >= max_row)
					vs_scroll();

					/* the remainder of the tab */
				VCOL = tab_stop - max_col;
			}
			else {
				for (; VCOL<tab_stop; VCOL++)
					LVS[VROW][VCOL] = ' ';
			}
			break;
		case '\r':		/* carriage return */
			VCOL = 0;
			if (!status->add_lf)
				break;
			/* fall thru...*/
		case '\n':		/* line feed */
			VROW++;
			if (VROW >= max_row)
				vs_scroll();
			break;
		default:		/* a normal character */
			LVS[VROW][VCOL] = c;
			VCOL++;
					/* wrap around */
			if (VCOL >= max_col) {
				VCOL = 0;
				VROW++;
				if (VROW >= max_row)
					vs_scroll();
			}
			break;
	}
	return(0);
}

#ifndef SHAREDMEM
/*
 * Save the virtual screen to a file.
 */

int
write_vs()
{
	FILE *fp1;
	register int i;

	if (!(fp1 = fopen(status->vs_path, "wb")))
		return(1);
					/* current x y coordinates */
	fprintf(fp1, "%d,%d\n", VROW, VCOL);

	for (i=0; i<max_row; i++) {
		LVS[i][max_col] = '\0';
		fprintf(fp1, "%s\n", LVS[i]);
	}
	fclose(fp1);
	return(0);
}

/*
 * Get the virtual screen image from the file.  Since input() gets
 * killed from time to time, the vs_path file is the only way to retain
 * the screen image.
 */

int
read_vs()
{
	FILE *fp;
	register int i;
	char buf[10];
				/* in case the fopen fails... */
	VROW = 0;
	VCOL = 0;
				/* not guaranteed to exist yet */
	if (!(fp = fopen(status->vs_path, "r")))
		return(1);
				/* get the x, y coordinates */
	fgets(buf, 10, fp);
	sscanf(buf, "%d,%d\n", &VROW, &VCOL);

				/* read the file into the vs array */
	for (i=0; i<max_row; i++) {
		fgets(LVS[i], MAX_COL+2, fp);
		LVS[i][max_col] = '\0';
	}
	fclose(fp);
	return(0);
}
#endif /* SHAREDMEM */

/*
 * If the user clears the screen with the ^A-C command, the input
 * has to be in sync.
 */

int
vs_clear()
{
	register int j, i;

	for (i=0; i < max_row; i++) {
		LVS[i][max_col] = '\0';
		for (j=0; j<max_col; j++)
			LVS[i][j] = ' ';
	}
					/* home the "cursor" */
	VROW = 0;
	VCOL = 0;
	return(0);
}

/*
 * Do a software scroll on the virtual screen.  Does not alter the
 * "col" variable.
 */

int
vs_scroll()
{
	register int i;
	char *strcpy();
					/* move 'em up 1 line */
	for (i=0; i<max_row-1; i++)
		strcpy(LVS[i], LVS[i+1]);
					/* clear the bottom line */
	for (i=0; i<max_col; i++)
		LVS[max_row-1][i] = ' ';

	VROW = max_row -1;
	return(0);
}

/*
 * Do a buffered read from the serial port.
 */

int
readbyte(byte)
int *byte;
{
	static unsigned char ibuf[CLIST];
	static unsigned char *bufp = ibuf;
	static int n;
	static int	allbytes;

	int rc, erg;
	rc = erg = 0;
	*byte = (unsigned char *)NULL;

	if (n <= 0) {
		if ((n = read(status->fd, ibuf, CLIST)) > 0)
		{
			bufp = ibuf;
		}
		else
		{
			if(DEBUG) fprintf(flink, "<rc=%d>", n);
		}
	}
	if (n > CLIST)
	{
		fprintf(flink,"\n<RD-ERROR[%d]>\n",n);
		return( 0 );	
	}
	if (n > 0)
	{
		erg = (int) (*bufp++ & 0xff);
		allbytes++;
		rc = 1;
		n--;
	}
	*byte = erg;
	return(rc);
}
