/*
 * Routines to manipulate the pcomm.param file.
 */

#include <stdio.h>
#include "config.h"
#include "param.h"

/*
 * Read the parameter structure from the pcomm.param file.  Returns a
 * pointer to the PARAM structure.  All errors are fatal.
 */

struct PARAM *
read_param(extra)
char *extra;
{
	FILE *fp;
	int i, oops;
	char buf[80], *temp_token, *str, *strdup(), *findfile();
	char message[80], *str_tok();
	static char *token[NUM_PARAM] = {"D_BAUD", "D_PARITY", "D_DBITS",
	"D_SBITS", "HOT", "ASCII_HOT", "D_DUPLEX", "FLOW", "CR_IN", "CR_OUT",
	"LOGFILE", "DUMPFILE", "STRIP", "PAUSE_CHAR", "CR_CHAR", "CTRL_CHAR",
	"ESC_CHAR", "BRK_CHAR", "ABORT", "C_DELAY", "R_DELAY", "LECHO",
	"EXPAND", "CR_DELAY", "PACE", "CR_UP", "LF_UP", "TIMER", "CR_DN",
	"LF_DN", "LD_PLUS", "LD_MINUS", "LD_AT", "LD_POUND", "MAC_1",
	"MAC_2", "MAC_3", "MAC_4", "MAC_5", "MAC_6", "MAC_7", "MAC_8",
	"MAC_9", "MAC_0", "BMODE", "BID", "BMN", "BPW", "LANG"};
	static struct PARAM p;
	void error_win();

	if ((p.p_path = findfile(extra, "pcomm.param")) == (char *)NULL)
		fprintf(stderr,"Support file 'pcomm.param' is missing");
					/* read permission already checked */
	fp = fopen(p.p_path, "r");

	oops = 0;
	for (i=0; i<NUM_PARAM; i++) {
		if (fgets(buf, 80, fp) == (char *)NULL) {
			sprintf(message, "is truncated at line %d", i+1);
			oops++;
			break;
		}
					/* parse the input line */
		if (!(temp_token = str_tok(buf, '='))) {
			sprintf(message, "is missing a token at line %d", i+1);
			oops++;
			break;
		}
		if (!(str = str_tok((char *) NULL, '\n'))) {
			sprintf(message,"missing a parameter at line %d",i+1);
			oops++;
			break;
		}
					/* sanity checking */
		if (strcmp(temp_token, token[i])) {
			sprintf(message, "is corrupted at line %d", i+1);
			oops++;
			break;
		}

		switch (i) {
					/* used in ls_menu() */
			case LINE_SET:
				p.d_baud = atoi(str);
				break;
			case LINE_SET+1:
				p.d_parity = *str;
				break;
			case LINE_SET+2:
				p.d_dbits = atoi(str);
				break;
			case LINE_SET+3:
				p.d_sbits = atoi(str);
				break;

					/* used in term_setup() */
			case TERM_SETUP:
				p.hot = atoi(str);
				break;
			case TERM_SETUP+1:
				p.ascii_hot = strdup(str);
				break;
			case TERM_SETUP+2:
				p.d_duplex = strdup(str);
				break;
			case TERM_SETUP+3:
				p.flow = strdup(str);
				break;
			case TERM_SETUP+4:
				p.cr_in = strdup(str);
				break;
			case TERM_SETUP+5:
				p.cr_out = strdup(str);
				break;

					/* used in gen_setup() */
			case GEN_SETUP:
				p.logfile = strdup(str);
				break;
			case GEN_SETUP+1:
				p.dumpfile = strdup(str);
				break;
			case GEN_SETUP+2:
				p.strip = strdup(str);
				break;
			case GEN_SETUP+3:
				p.pause_char = *str;
				break;
			case GEN_SETUP+4:
				p.cr_char = *str;
				break;
			case GEN_SETUP+5:
				p.ctrl_char = *str;
				break;
			case GEN_SETUP+6:
				p.esc_char = *str;
				break;
			case GEN_SETUP+7:
				p.brk_char = *str;
				break;
			case GEN_SETUP+8:
				p.abort = strdup(str);
				break;

					/* used in gen_setup() delay_times() */
			case DELAY_TIMES:
				p.c_delay = atoi(str);
				break;
			case DELAY_TIMES+1:
				p.r_delay = atoi(str);
				break;

					/* used in axfer_setup() */
			case ASCII_SETUP:
				p.lecho = strdup(str);
				break;
			case ASCII_SETUP+1:
				p.expand = strdup(str);
				break;
			case ASCII_SETUP+2:
				p.cr_delay = atoi(str);
				break;
			case ASCII_SETUP+3:
				p.pace = strdup(str);
				break;
			case ASCII_SETUP+4:
				p.cr_up = strdup(str);
				break;
			case ASCII_SETUP+5:
				p.lf_up = strdup(str);
				break;
			case ASCII_SETUP+6:
				p.timer = atoi(str);
				break;
			case ASCII_SETUP+7:
				p.cr_dn = strdup(str);
				break;
			case ASCII_SETUP+8:
				p.lf_dn = strdup(str);
				break;

					/* used in d_revise() */
			case LD_CODES:
				p.ld_plus = strdup(str);
				break;
			case LD_CODES+1:
				p.ld_minus = strdup(str);
				break;
			case LD_CODES+2:
				p.ld_at = strdup(str);
				break;
			case LD_CODES+3:
				p.ld_pound = strdup(str);
				break;

					/* used in macro() */
			case MACROS:
				p.mac_1 = strdup(str);
				break;
			case MACROS+1:
				p.mac_2 = strdup(str);
				break;
			case MACROS+2:
				p.mac_3 = strdup(str);
				break;
			case MACROS+3:
				p.mac_4 = strdup(str);
				break;
			case MACROS+4:
				p.mac_5 = strdup(str);
				break;
			case MACROS+5:
				p.mac_6 = strdup(str);
				break;
			case MACROS+6:
				p.mac_7 = strdup(str);
				break;
			case MACROS+7:
				p.mac_8 = strdup(str);
				break;
			case MACROS+8:
				p.mac_9 = strdup(str);
				break;
			case MACROS+9:
				p.mac_0 = strdup(str);
				break;
			case BMODE:
				p.btx	= atoi(str);
				break;
			case BID:
				p.btx_id = strdup(str);
				break;
			case BMN:
				p.btx_mn = strdup(str);
				break;
			case BPW:
				p.btx_pw = strdup(str);
				break;
			case LANG:
				p.lang = strdup(str);
				break;
		}
	}
	fclose(fp);
	if (oops) {
		sprintf(buf, "Parameter file '%s'", p.p_path);
		fprintf(stderr, buf);
	}
	return(&p);
}

/*
 * Search the extra directory (supplied on the command line), then the
 * directory in the PCOMM environmental variable, then the current working
 * directory, and lastly, the default directory.
 */

char *
findfile(extra, name)
char *extra, *name;
{
	int i;
	char *pcomm, *getenv(), *path, pbuf[200], *getcwd(), *strdup();
	char temp[200];

					/* see if PCOMM variable is set */
	pcomm = getenv("PCOMM");
	if (pcomm == (char *)NULL || *pcomm == '\0')
		pcomm = (char *)NULL;
	else {
					/* zap the trailing separator */
		if (pcomm[strlen(pcomm)-1] == '/')
			pcomm[strlen(pcomm)-1] = '\0';
	}

	for (i=0; i<4; i++) {
					/* directory search order */
		switch (i) {
			case 0:		/* extra directory from command line */
				path = extra;
				break;
			case 1:		/* PCOMM environmental variable */
				path = pcomm;
				break;
			case 2:		/* current working directory */
				path = getcwd(pbuf, 200);
				break;
			case 3:		/* Pcomm's default directory */
				path = DEFAULT_DIR;
				break;
		}
		if (path == (char *)NULL)
			continue;

		sprintf(temp, "%s/%s", path, name);
					/* read permission checked */
		if (!access(temp, 4))
			return(strdup(temp));
	}
	return( (char *)NULL);
}

char *
str_tok(str, c)
char *str, c;
{
	char *strchr();
	static char *ptr, *sep;
					/* start at beginning */
	if (str != (char *)NULL)
		ptr = str;
	else
		ptr = sep;
					/* at the end? */
	if (*ptr == '\0')
		return( (char *)NULL);
					/* no separator? */
	if (!(sep = strchr(ptr, c)))
		return( (char *)NULL);
					/* zap the sep, move past it */
	*sep = '\0';
	sep++;

	return(ptr);
}
