/*
 * Start the terminal dialogue, fork the input routine, watch for the
 * hot key so we can execute an option.
 */
#include <time.h>
#include <stdio.h>
#include <curses.h>
#include <signal.h>
#include <errno.h>

#include "config.h"
#ifdef UNIXPC
#include <sys/phone.h>
#include <fcntl.h>
#endif /* UNIXPC */
#include "dial_dir.h"
#include "misc.h"
#include "modem.h"
#include "param.h"
#include "status.h"
#include "btx.h"

static int pid = -1;

terminal(input_status)
int input_status;
{
	extern int fd;
	int k, cr_lf, rc;
	int	i=0;
	char c=0, lf=10, *strdup(), *init, *read_codes();
	int	cr=0;
	void help_screen(), line_set(), n_shell(), load_vs(), send_str();
	void system_screen(), release_port(), list_dir(), pexit(), zap_vs();
	void st_line(), chg_dir(), screen_dump(), input_off(), suspend();
	void show_calls(), info(), term_mode(), error_win(), perror();
	int  do_input(), shutdown();
	unsigned short btx_init();
	unsigned int	sleep();
	char	*btx_fname();
	int	btx_send(), btx_end();
	int	btx_BATCH=OFF;		/* no inputs when batch mode ON	*/
	long	akttime=0L;
	long	batchstart=0L;

	time(&batchstart);		/* init starttime for batch-mode */
				
					/* if starting out in command mode */
	if ((!input_status) && (!btx_CRON))
		st_line("");

					/* put stdin/stdout in terminal mode */
	resetterm();
	term_mode();
	cr_lf = !strcmp(param->cr_out, "CR/LF");

	strcpy(status->log_macro, btx_MFILE);	/* set default filename */
	
	if (input_status){
		rc=do_input( status->btx );
	}

	if(btx_MACRO)
		signal(SIGCLD,btx_end);	/* program end when child is end */
	else
		signal(SIGCLD,SIG_IGN);


	while (1) {
		if(btx_MACRO) {			/* auto-macro start -bc */
			c=param->hot;
			status->log = ON;	/* set logfile on	*/
			strcpy(status->log_path, btx_fname(btx_MFILE));
			strcpy(status->log_macro, btx_MFILE);
			dir->d_cur  = btx_MACRO; /* set dial entry num 	*/
			btx_BATCH   = ON;
		}else{
			/* when Batch Macro-Mode then no key response	*/
			if(!btx_CRON){
				read(0, &c, 1);
				cr = c;
				c &= 0x7f;
			}else{	/* check batch timeout 20 min.		*/
				time(&akttime);
				if(akttime > batchstart + 660){
					error_win(1,"Cron Batch-Mode has timeout  at 11 minute","");
					c=param->hot;	/* select hot */
					i='H';
				}
			}
		}
					/* is it the hot key? */
		if (c == param->hot) {
					/* suspend input */
			input_status = 0;
			suspend(1);

			/*
			 * Put in terminal in the curses mode, load the
			 * virtual screen and add the status line at the bottom.
			 */
			fixterm();
			load_vs();
			st_line(" HOT ");
	
			if(btx_MACRO){	/* automatic dial for batch mode */
				i='D';
				c=NUL;
			}else{	
#ifndef OLDCURSES
			/*
				if(!btx_CRON)
					keypad(stdscr, TRUE);
			*/
#endif /* OLDCURSES */
				if(!btx_CRON)
					i = wgetch(stdscr);
			}
					/* map an additional hot key to -1 */
			if (i == param->hot){
				i = -1;
			}
					/* look for options */
			k = -1;

			switch (i) {
				case -1:	/* 2 "hots" means send 1 */
					k = param->hot;
					break;
				case '0':	/* help screen */
					help_screen(param->ascii_hot);
					st_line(" OK ");
					break;
				case '<':	/* show log-list	*/
					show_calls();
					st_line(" OK ");
					break;
				case '>':
					system_screen();
					st_line(" OK ");
					break;
				case 'd':
				case 'D':	/* dialing directory */
					if(dial_menu()){
						time(&START_TIME);
						if(dir->duplex[dir->d_cur]=='B'){
							if( btx_init() ){
								btx_MACRO=OFF;
								break;
								/* end batch */
							}		
							if(!btx_TERM){
								status->btx=3;
							}else{
								status->btx=1;
							}
							if(btx_CRON){
								status->btx=2;
							}	
						}else{
							status->btx=0;
						}
						input_status = dial_win();
					}
					if(btx_CRON){
						c=0;
						i=0;
					}
					btx_MACRO=OFF;
					break;
				case 'r':
				case 'R':	/* redial */
					if( redial() )
						input_status = dial_win();
					btx_MACRO=OFF;
					st_line(" OK ");
					break;
				case 'm':
				case 'M':	/* keyboard macros */
					macro();
					st_line(" OK ");
					break;
				case 'p':
				case 'P':	/* line settings */
					if (ls_menu())
						line_set();
					break;
				case 'x':
				case 'X':	/* exit */
					pexit(1);
					break;
				case '4':	/* Unix gateway */
					n_shell();
					break;
				case 'i':
				case 'I':	/* Program info screen */
					info(0);
					st_line(" OK ");
					break;
				case 's':	/* setup menu */
				case 'S':
					input_status = setup_menu();
					st_line(" SETUP ");
					break;
				case 'c':	/* clear the screen */
				case 'C':
					zap_vs();
					erase();
					st_line(" OK ");
					break;
				case 'b':
				case 'B':	/* Change directory */
					chg_dir();
					st_line(" OK ");
					break;
				case 'e':
				case 'E':	/* toggle duplex */
					if (dir->duplex[dir->d_cur] == 'F')
						dir->duplex[dir->d_cur] = 'H';
					else
						dir->duplex[dir->d_cur] = 'F';

						/* show changes */
					st_line(" LINE ");
					k = wait_key(stdscr, 2);
					break;
				case 'h':
				case 'H':	/* hang up phone */
					if(!btx_CRON)	st_line(" HANGUP ");
					release_port(1);
					input_off();
					if(!btx_CRON)	st_line(" OK ");
					btx_MACRO=OFF;
					break;
				case 'z':
				case 'Z':
					st_line(" INIT ");
					line_set();
					init="ATZ\r";
					send_str( init );
#ifdef	WELLCON
					sleep(2);
					init="ATQ0\r";
					send_str( init );
					init=read_codes(3);
#endif
					st_line( init );
					break;
				case 'q':
				case 'Q':
					LANGUAGE = LANGUAGE ? 0 : 1;
					if(LANGUAGE)	st_line(" DEUTSCH ");
					else		st_line(" ENGLISH ");
				case 'l':
				case 'L':	/* toggle printer */
					if(!status->btx){
					status->print = status->print ? 0 : 1;
#ifndef SHAREDMEM
					if (pid != -1)
						kill(pid, SIGUSR2);
#endif /* SHAREDMEM */
						/* show changes */
					st_line(" LPR ");
					k = wait_key(stdscr, 2);
					}
					break;
				case '3':	/* toggle CR - CR/LF */
					if (!strcmp(param->cr_in, "CR")) {
						param->cr_in = strdup("CR/LF");
						status->add_lf = 1;
					}
					else {
						param->cr_in = strdup("CR");
						status->add_lf = 0;
					}
#ifndef SHAREDMEM
					input_off();
					input_status++;
#endif /* SHAREDMEM */
						/* show changes */
					st_line("");
					k = wait_key(stdscr, 2);
					break;
				case '7':	/* break key */
					if (fd != -1)
						ioctl(fd, TCSBRK, 0);

					st_line(" BREAK ");
					break;
#ifndef OLDCURSES
				case KEY_UP:
#endif /* OLDCURSES */
				case 'u':
				case 'U':	/* send files */
					input_status = xfer_menu(1);
					break;
#ifndef OLDCURSES
				case KEY_DOWN:
				case '\n':
#endif /* OLDCURSES */
				case 'n':
				case 'N':	/* receive files */
					input_status = xfer_menu(0);
					break;
				case 't':
				case 'T':
					input_status = pass_thru();
					break;
				case 'f':
				case 'F':	/* list directory */
					list_dir();
					st_line(" TIME ");
					break;
				case 'g':	/* screen dump */
				case 'G':
					screen_dump();
					st_line(" DUMP ");
					k = wait_key(stdscr, 2);
					break;
				case '1':	/* data logging */
					input_status = data_logging();
					break;
				case '2':	/* toggle log */
					if(!strcmp(status->log_path,"NONE")){
						beep();
						st_line(" LOG ");
						k = wait_key(stdscr, 2);
						break;
					}
					status->log = status->log ? 0 : 1;
#ifndef SHAREDMEM
					if (pid != -1)
						kill(pid, SIGUSR1);
#endif /* SHAREDMEM */
						/* show changes */
					st_line("");
					k = wait_key(stdscr, 2);
					break;
				/*
				 * The following are the keyboard macros
				 * corresponding to the shifted number keys.
				 * (Too many keys... [control] [A] [shift] [1]
				 * is hardly a shortcut!)
				 */
				case '!':
					send_str(param->mac_1);
					break;
				case '@':
					send_str(param->mac_2);
					break;
				case '#':
					send_str(param->mac_3);
					break;
				case '$':
					send_str(param->mac_4);
					break;
				case '%':
					send_str(param->mac_5);
					break;
				case '^':
					send_str(param->mac_6);
					break;
				case '&':
					send_str(param->mac_7);
					break;
				case '*':
					send_str(param->mac_8);
					break;
				case '(':
					send_str(param->mac_9);
					break;
				case ')':
					send_str(param->mac_0);
					break;
				default:
					putchar( (int)BEL);
					st_line(" UNDO ");
					break;
			}

			/*
			 * Repaint the stdscr (if we are already talking),
			 * get the stdin/stdout out of the curses mode and
			 * into the terminal mode.
			 */
			if (fd != -1) {
				touchwin(stdscr);
				refresh();
			}
			if(!status->btx){
				resetterm();
				term_mode();
			}

			/*
			 * Some of the output processing options have to be
			 * faked...  Unfortunately, adding a LF to CR on
			 * output is one of them.
			 */
			cr_lf = !strcmp(param->cr_out, "CR/LF");

					/* re-start input routine */
			if (input_status){
				rc=do_input( status->btx );
			}else{
				suspend(0);
			}		

			/*
			 * If you pressed a key during one of the sleeping
			 * periods (typically the delay to see the status
			 * line change), let the keyboard value fall thru
			 * to the write() below.
			 */
			if (k == -1)
				continue;
			c = k;
		}

		if(status->btx){
			c = btx_send(cr);
		}
	
					/* ignore errors if fd == -1 */
		if((!btx_CRON) && (c)){
			write(fd, &c, 1);
		}
					/* map cr to cr_lf? */
		if ((c == '\r' && cr_lf) && (!status->btx))
			write(fd, &lf, 1);
	}
}

/*
 * Put the stdin/stdout in terminal mode.  We've divided up the
 * responsibility for the line settings options between the serial port
 * and the stdin and stdout.
 */

void
term_mode()
{
	struct termio tbuf;

	ioctl(0, TCGETA, &tbuf);

	tbuf.c_cc[4] = 1;		/* VMIN */
	tbuf.c_cc[5] = 0;		/* VTIME */
	tbuf.c_iflag = 0;
	tbuf.c_oflag = 0;
	tbuf.c_lflag = 0;
					/* half duplex */
	if (dir->duplex[dir->d_cur] == 'H')
		tbuf.c_lflag = ECHO;

	ioctl(0, TCSETA, &tbuf);
	ioctl(0, TCFLSH, 2);
	return;
}

/*
 * Fire up the input routine...
 */

static int
do_input( mode )
int mode;
{
	extern int fd;
	void error_win();
	char first[(sizeof(int)*8)+1];
	int	rc=-1;
#ifdef SHAREDMEM
	extern int shm_id;
#else /* SHAREDMEM */
	char add_lf[2], log[2], print[2], zbtx[2];
#endif /* SHAREDMEM */
					/* if no TTY, or already on */
	if (pid != -1 || fd == -1)
		return;

	status->fd = fd;
	if (!strcmp(param->cr_in, "CR/LF"))
		status->add_lf = 1;
	else
		status->add_lf = 0;

#ifdef SHAREDMEM
	sprintf(first, "%d", shm_id);
#else /* SHAREDMEM */
	sprintf(first, "%d", status->fd);
	sprintf(add_lf, "%d", status->add_lf);
	sprintf(log, "%d", status->log);
	sprintf(print, "%d", status->print);
	sprintf(zbtx, "%01d", mode);
#endif /* SHAREDMEM */

					/* fork the input routine */
	if (!(pid = fork()) ) {
		if( pid < 0){
			error_win(1, "Can't start child process","");
			exit(-1);
		}
		if( setpgrp() < 0){
			error_win(1, "Can't set process-group-id","");
			exit(-1);
		}
#ifdef SETUGID
		if( setuid(getuid()) < 0){
			error_win(1,"Can't set userid","");
			exit(-1);
		}
		if( setgid(getgid()) < 0){
			error_win(1,"Can't set group-id","");
			exit(-1);
		}
#endif /* SETUGID */

#ifdef SHAREDMEM
		rc=execlp("pcomm_input", "pcomm_input", first, (char *) 0);
#else /* SHAREDMEM */
		rc=execlp("pcomm_input", "pcomm_input", first, add_lf, log,
		print, zbtx , status->log_path, status->log_macro, 
		status->vs_path, (char *) 0);
#endif /* SHAREDMEM */
		fprintf(stderr, "[error fork pid=%d rc=%d]\n", pid, rc);
		perror("fork");
		error_win(1,"Can't execute the 'pcomm_input' program", "");
	}
	return(rc);
}

/*
 * shut it down...
 */

void
input_off()
{
	if (pid != -1) {
		/* kill when in use	*/
		kill(pid, SIGTERM);
		pid = -1;
	}
	return;
}

/*
 * Hang up the phone but remain in the Pcomm command state.  Uses the
 * hang_up string only, does *not* drop the DTR!
 */

void
hang_up(verbose)
int verbose;
{
	FILE	*log, *fopen();

	struct tm *tm_now;
	char	ausgabe[81];
	long	minuten=0L;
	float	kosten=0L;
	long	einheiten=0L;
	char	typ[10];
	int	z,distanze=0;
	char	*getlogin();
	char	*read_codes();
	extern int fd;
	void send_str(), st_line(), line_set();
#ifdef UNIXPC
	char buf[40], *strcpy(), *ttyname();
#endif /* UNIXPC */

	time(&END_TIME);
	tm_now  = localtime(&START_TIME);
	minuten = (END_TIME - START_TIME) / 60;
	for(z=0; z<strlen(dir->number[dir->d_cur]); z++){
		if(dir->number[dir->d_cur][z]=='-')	distanze=1;
	}	
	if((tm_now->tm_hour >= 18) || (tm_now->tm_hour <= 8)){
		if(!distanze){	
			kosten    = (minuten / 12  ) * 0.23 + 0.23;
			einheiten = (minuten / 12  ) + 1;
		}else{
			kosten    = (minuten / 0.4 ) * 0.23 + 0.23;
			einheiten = (minuten / 0.4 ) + 1;
		}
	}else{
		if(!distanze){
			kosten    = (minuten / 8   ) * 0.23 + 0.23;
			einheiten = (minuten / 8   ) + 1;
		}else{
			kosten    = (minuten / 0.25) * 0.23 + 0.23;
			einheiten = (minuten / 0.25) + 1;
		}
	}
	if(strcmp(btx_MFILE,"NONE") == 0)
		strcpy(typ,"ONLINE");
	else
		strcpy(typ,"BATCH ");
	
					/* sanity checking */
	if (modem == NULL)
		return;
					/* anything to hang up? */
	if (modem->m_cur == -1 || fd == -1)
		return;

	sprintf(ausgabe,"%s [%8s] [%14s] %02d.%02d.%02d  %02d:%02d %03ld %03ld %06.2f %s\n",
			 typ, getlogin(),dir->number[dir->d_cur],
			 tm_now->tm_mday, tm_now->tm_mon+1, tm_now->tm_year,
			 tm_now->tm_hour, tm_now->tm_min,
			 minuten, einheiten, kosten, btx_MFILE);
	if((log=fopen("/usr/spool/uucp/pcomm.calls","a"))==(FILE *)NULL){
		if(!btx_CRON) error_win(0,"Can't open logfile /usr/spool/uucp/phone.calls","");
		else fprintf(stderr, "Can't open logfile /usr/spool/uucp/phone.calls");	
	}else{
		fprintf( log, "%s", ausgabe);
		fclose( log );
	} 
	if (verbose)
		st_line(" HUCL");
					/* special case for OBM */
#ifdef UNIXPC
	if (!strcmp(modem->mname[modem->m_cur], "OBM")) {
		ioctl(fd, PIOCDISC);
		/*
		 * The PIOCDISC ioctl screws up the file descriptor!!!
		 * No other phone(7) ioctl can fix it.  Whatever it does,
		 * it seems to escape detection with PIOCGETA and TCGETA.
		 * The best I can do is close the port and start over.
		 */
		strcpy(buf, ttyname(fd));
		close(fd);
		fd = open(buf, O_RDWR|O_NDELAY);
		line_set();
		fcntl(fd, F_SETFL, fcntl(fd, F_GETFL, 0) & ~O_NDELAY);
	}
	else
#endif /* UNIXPC */
		send_str(modem->hang_up[modem->m_cur]);
#ifdef	WELLCON
		if(strcmp( read_codes(3), "OK")==0){
			st_line(" H-OK ");
			return;
		}
#endif
	if (verbose)
		st_line(" HU ");
	return;
}

/*
 * Suspend or un-suspend the input routine.  The argument is used in
 * non-shared memory configurations to give the vs_path file a fighting
 * chance of being written to disk before load_vs() reads it.
 */

/*ARGSUSED*/
void
suspend(on)
int on;
{
	unsigned int sleep();

	if (pid == -1)
		return;
	kill(pid, SIGINT);
#ifndef SHAREDMEM
	if (on)
		sleep(1);
#endif /* SHAREDMEM */
	return;
}

char	*btx_fname( fname ) 
char	fname[10];
{  
static char	*datum; 
struct 	tm 	*tm_now;  
char		*nfname;
long secs_now;  
int	i; 
extern	char	*malloc();

if(nfname==(char *)NULL) nfname=malloc(10); 
if(datum==(char *)NULL)	 datum=malloc(80);
 
if(fname==(char *)NULL)
	strncpy(nfname,"NONE",4);
else
	strncpy(nfname,fname, 4);

time(&secs_now);  
tm_now=localtime(&secs_now);  
 
sprintf(datum, "%4.4s_%02d%02d%02d.%02d", 
  nfname,tm_now->tm_mday, tm_now->tm_mon+1, tm_now->tm_year,tm_now->tm_hour); 

return( datum );   
 
} 
