/*
 * Routines to support the batch protocols.
 */

#include <stdio.h>
#include <ctype.h>
#include <curses.h>
#include "config.h"
#include "misc.h"
#include "status.h"
#include "xmodem.h"
#include "btx.h"

/*
 * Send the file name for the tarsparent mode batch.  
 * Transparent-Mode is define by BTX-CEPT standard
 */


int
send_trans(win, name, rfp)
WINDOW *win;
char *name;
FILE *rfp;
{
	FILE	*lfp, *fopen();
	unsigned short sum, calc_crc16();
	int	send_trans_block();
	unsigned char	crc_1, crc_2;

	unsigned	char	buffer[1024];
	unsigned	char	sblock[1024];
	unsigned	char	answ;
	int	block_size=256;

	int	z,k,anzahl,i=0;
	int	block_count=0;
	int	bytes_count=0;
	int	error_count=0;
	int	err_nak=0;
	int	rc=0;
	
	if( (lfp=fopen("tran.log","w"))==(FILE *)NULL){
		fprintf(stderr, "Can't open tran.log");
		return(CANCEL);
	}

	putc_line(EOT); 		/* optional initialisierung 	*/

	putc_line(DLE);			/* begin transparent mode	*/
	putc_line(ENQ);


	if( (answ=getc_line(20)) == DLE){ /* warte auf antwort von VSt	*/
		fprintf(lfp, "<ANSW0[%x]>", answ);

		if( (answ=getc_line(5)) == ACK0){
			fprintf(lfp, "<ANSW1[%x]>", answ);
			clear_line(win, 12, 24, 1);
			waddstr(win, "INIT OK");
			wrefresh(win);
			sleep(1);
		}else{
			mvwprintw(win,12,24,"INIT ERROR ACK");
			wrefresh(win);
			return(ERROR);
		}
	}else{
		mvwprintw(win,12,24,"KEINE ANWORT");
		wrefresh(win);
		return(ERROR);
	}
	
	clear_line(win,  7,24,1 );
	mvwprintw(win, 5,24,"BCC-CHECK");
	mvwprintw(win, 7,24,"%-3d", block_count);
	mvwprintw(win, 9,24,"%-3d", bytes_count);
	mvwprintw(win,12,24,"START TRANSMIT");
	wrefresh(win);

	while(err_nak < 10){	
		if(!err_nak)
	           anzahl=fread((char *)&buffer[2],sizeof(char),block_size,rfp);
	
		if(anzahl <= 0){
			clear_line(win,12,24,1);
			mvwprintw(win,12,24,"TRANSFER OK");
			putc_line(EOT);
			return(1);
		}
	
		fprintf(lfp, "<READ %d BYTES>\n", anzahl);

		buffer[anzahl+3]    =ETB;	/* zusammenhaengender block */
		if(anzahl < block_size)
			buffer[anzahl+3]=ETX;

		buffer[0]	    =DLE;
		buffer[1]	    =STX;
		buffer[anzahl+2]    =DLE;
		k=0;
		for(z=2; z < anzahl+2; z++){
			sblock[k]   = buffer[z];
			fprintf(lfp,"(%x)",sblock[k]);
			k++;
		}
		sblock[k]=buffer[anzahl+3];
		fprintf(lfp,"\n(%x)%d\n", sblock[k], k);
		k++;	
		sum = calc_crc16( sblock , k );
		crc_1 = sum >> 8;
		crc_2 = sum;

		buffer[anzahl+4]=crc_2;
		buffer[anzahl+5]=crc_1;
		buffer[anzahl+6]='\0';

		fprintf(lfp, "<crc1=%x crc2=%x>\n", crc_1, crc_2);

		/* send him DLE STX data ........ DLE ETB BCC BCC	*/
		if(err_nak != -1)
			rc=send_trans_block(buffer, anzahl+6, lfp);

		i = getc_line(10);
		fprintf(lfp, "<ANSW2[%x]>", i);
		if( i == DLE ){
			i=getc_line(3);
			fprintf(lfp, "<ANSW3[%x]>", i);
		}

		fprintf(lfp, "<ANSW4[%x]>", i);

		switch(i) {
			case   -1:
				err_nak=-1;
				error_count++;
				clear_line(win, 12, 24, 1);
				waddstr(win, "KEINE ANTWORT");
				wrefresh(win);
				sleep(1);
				putc_line(ENQ);
				break;
			case  EOT:
				clear_line(win, 12, 24, 1);
				waddstr(win, "ABBRUCH (VST)");
				mvwprintw(win, 11,24,"%-3d", error_count);
				wrefresh(win);
				return(ERROR);
				break;
			case  ENQ:
				error_count++;
				err_nak++;
				clear_line(win, 12, 24, 1);
				waddstr(win, "FEHLER ENQ");
				wrefresh(win);
				putc_line(EOT);
				break;
			case ACK0:
			case ACK1:
				block_count++;
				bytes_count = bytes_count + anzahl;
				clear_line(win, 7, 24, 1);
				mvwprintw(win,  7, 24,"%-3d", block_count);
				mvwprintw(win,  9, 24,"%-3d", bytes_count);
				wrefresh(win);
				err_nak=0;
				break;
			case NAK:
				error_count++;
				err_nak++;
				clear_line(win, 12, 24, 1);
				waddstr(win, "WIEDERHOLEN");
				mvwprintw(win,  10, 24,"%-3d", err_nak);
				mvwprintw(win,  11, 24,"%-3d", error_count);
				wrefresh(win);
				putc_line(EOT);
				sleep(1);
				break;
			default:
				clear_line(win, 12, 24, 1);
				waddstr(win, "TIME OUT");
				mvwprintw(win, 12, 35,"%x",i); 
				wrefresh(win);
				error_count++;
				err_nak++;
				sleep(1);
				/*	
				putc_line(EOT);
				return(ERROR);
				*/
				break;
		}
	}
	return(1);
}

int
send_trans_block( blk, paket ,lfp)
unsigned char	blk[1024];
unsigned int	paket;
FILE	*lfp;
{
int	z;
	for(z=0; z < paket; z++){
		putc_line( blk[z] );
		fprintf(lfp,"[%x]", blk[z]);
	}
	return(z);
}
